]> git.sesse.net Git - nageru/blobdiff - mixer.h
Rework entire pts handling.
[nageru] / mixer.h
diff --git a/mixer.h b/mixer.h
index b2561d15c0b395f2d82e2a5d92b682b7ab634942..45311aa42ec98af08cf7fb55719c16e3e01c0041 100644 (file)
--- a/mixer.h
+++ b/mixer.h
@@ -9,12 +9,14 @@
 #include <movit/flat_input.h>
 #include <functional>
 
-#include "bmusb.h"
+#include "bmusb/bmusb.h"
 #include "h264encode.h"
 #include "pbo_frame_allocator.h"
 #include "ref_counted_frame.h"
 #include "ref_counted_gl_sync.h"
 #include "theme.h"
+#include "resampler.h"
+#include "timebase.h"
 
 #define NUM_CARDS 2
 
@@ -79,7 +81,7 @@ public:
 
        std::vector<std::string> get_transition_names()
        {
-               return theme->get_transition_names(frame / 60.0);
+               return theme->get_transition_names(pts());
        }
 
 private:
@@ -90,6 +92,7 @@ private:
        void thread_func();
        void subsample_chroma(GLuint src_tex, GLuint dst_dst);
        void release_display_frame(DisplayFrame *frame);
+       double pts() { return double(pts_int) / TIMEBASE; }
 
        QSurface *mixer_surface, *h264_encoder_surface;
        std::unique_ptr<movit::ResourcePool> resource_pool;
@@ -101,7 +104,7 @@ private:
        // Effects part of <display_chain>. Owned by <display_chain>.
        movit::FlatInput *display_input;
 
-       int frame = 0;
+       int64_t pts_int = 0;  // In TIMEBASE units.
 
        std::mutex bmusb_mutex;
        struct CaptureCard {
@@ -112,10 +115,17 @@ private:
                QSurface *surface;
                QOpenGLContext *context;
 
-               bool new_data_ready = false;  // Whether new_frame contains anything.
+               bool new_data_ready = false;  // Whether new_frame and new_frame_audio contains anything.
+               bool should_quit = false;
                RefCountedFrame new_frame;
                GLsync new_data_ready_fence;  // Whether new_frame is ready for rendering.
+               std::vector<float> new_frame_audio;
                std::condition_variable new_data_ready_changed;  // Set whenever new_data_ready is changed.
+               unsigned dropped_frames = 0;  // Before new_frame.
+
+               std::mutex audio_mutex;
+               std::unique_ptr<Resampler> resampler;  // Under audio_mutex.
+               int last_timecode = -1;  // Unwrapped.
        };
        CaptureCard cards[NUM_CARDS];  // protected by <bmusb_mutex>